@@ -19,6 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "terminal-intl.h"
 #include "terminal-widget.h"
 
 #include <string.h>
@@ -28,8 +29,8 @@
 
 typedef struct
 {
-  int foo;
-  int skey_tag;
+  GSList *url_tags;
+  GSList *skey_tags;
 } VteData;
 
 static void
@@ -38,6 +39,9 @@ free_vte_data (gpointer data)
   VteData *vte;
 
   vte = data;
+
+  g_slist_free(vte->url_tags);
+  g_slist_free(vte->skey_tags);
   
   g_free (vte);
 }
@@ -53,6 +57,10 @@ terminal_widget_new (void)
   vte_terminal_set_mouse_autohide(VTE_TERMINAL(terminal), TRUE);
   
   data = g_new0 (VteData, 1);
+
+  data->url_tags = NULL;
+  data->skey_tags = NULL;
+
   g_object_set_data_full (G_OBJECT (terminal), "terminal-widget-data",
                           data, free_vte_data);
   
@@ -109,7 +117,13 @@ void
 terminal_widget_match_add                  (GtkWidget            *widget,
 					    const char           *regexp)
 {
-  vte_terminal_match_add(VTE_TERMINAL(widget), regexp);
+  VteData *data;
+  int tag;
+  
+  data = g_object_get_data (G_OBJECT (widget), "terminal-widget-data");
+
+  tag = vte_terminal_match_add (VTE_TERMINAL (widget), regexp);
+  data->url_tags = g_slist_append (data->url_tags, GINT_TO_POINTER(tag));
 }
 
 void
@@ -117,22 +131,28 @@ terminal_widget_skey_match_add             (GtkWidget            *widget,
 					    const char           *regexp)
 {
   VteData *data;
+  int tag;
   
   data = g_object_get_data (G_OBJECT (widget), "terminal-widget-data");
 
-  data->skey_tag = vte_terminal_match_add(VTE_TERMINAL(widget), regexp);
+  tag = vte_terminal_match_add(VTE_TERMINAL(widget), regexp);
+  data->skey_tags = g_slist_append (data->skey_tags, GINT_TO_POINTER(tag));
 }
 
 void
 terminal_widget_skey_match_remove          (GtkWidget            *widget)
 {
   VteData *data;
+  GSList *tags;
   
   data = g_object_get_data (G_OBJECT (widget), "terminal-widget-data");
 
-  vte_terminal_match_remove(VTE_TERMINAL(widget), data->skey_tag);
+  for (tags = data->skey_tags; tags != NULL; tags = g_slist_next(tags))
+    vte_terminal_match_remove(VTE_TERMINAL(widget),
+   			      GPOINTER_TO_INT(tags->data));
 
-  data->skey_tag = -1;
+  g_slist_free(data->skey_tags);
+  data->skey_tags = NULL;
 }
 
 char*
@@ -140,7 +160,20 @@ terminal_widget_check_match (GtkWidget *widget,
 			     int        column,
 			     int        row)
 {
-  return vte_terminal_match_check(VTE_TERMINAL(widget), column, row, NULL);
+  VteData *data;
+  GSList *tags;
+  gint tag;
+  char *match;
+   
+  data = g_object_get_data (G_OBJECT (widget), "terminal-widget-data");
+
+  match = vte_terminal_match_check(VTE_TERMINAL(widget), column, row, &tag);
+  for (tags = data->url_tags; tags != NULL; tags = g_slist_next(tags))
+    if (GPOINTER_TO_INT(tags->data) == tag)
+      return match;
+
+  g_free (match);
+  return NULL;
 }
 
 char*
@@ -149,14 +182,16 @@ terminal_widget_skey_check_match (GtkWidget *widget,
 				  int        row)
 {
   VteData *data;
+  GSList *tags;
   gint tag;
   char *match;
    
   data = g_object_get_data (G_OBJECT (widget), "terminal-widget-data");
 
   match = vte_terminal_match_check(VTE_TERMINAL(widget), column, row, &tag);
-  if (data->skey_tag == tag)
-    return match;
+  for (tags = data->skey_tags; tags != NULL; tags = g_slist_next(tags))
+    if (GPOINTER_TO_INT(tags->data) == tag)
+      return match;
 
   g_free (match);
   return NULL;
@@ -496,7 +531,18 @@ terminal_widget_fork_command (GtkWidget   *widget,
   *child_pid = vte_terminal_fork_command (VTE_TERMINAL (widget),
 		 			  path, argv, envp, working_dir,
 					  update_records, TRUE, TRUE);
-  return (*child_pid != -1);
+
+  if (*child_pid == -1)
+    {
+      g_set_error (err,
+                   G_SPAWN_ERROR,
+                   G_SPAWN_ERROR_FAILED,
+                   _("There was an error creating the child process for this terminal")
+                   );
+      return FALSE;
+    }
+
+  return TRUE;
 }
 
 
@@ -506,7 +552,7 @@ terminal_widget_get_estimated_bytes_per_scrollback_line (void)
 {
   /* One slot in the ring buffer, plus the array which holds the data for
    * the line, plus about 80 vte_charcell structures. */
-  return sizeof(gpointer) + sizeof(GArray) + (80 * (sizeof(wchar_t) + 4));
+  return sizeof(gpointer) + sizeof(GArray) + (80 * (sizeof(gunichar) + 4));
 }
 
 void
